# Parameters & Decision Variables
param Lx; param Ly; # workspace dimension in x- and y-direction
param Nx; param Ny; # number of neurons along the x- and y-axis
param dX := Lx/Nx; param dY := Ly/Ny; # cell size along the x- and y-axis
param Nh; param h; # time horizon and time step
param pi := 4*atan(1);  # 3.14159...
param Z{iy in 1..Ny, ix in 1..Nx}; # neural activity at each node
param psi0; param x0; param y0; # initial heading and position
param xT; param yT; # position of the target
param U; # forward velocity
param rmax; # Maximum turning rate
param s := 1-1e-5; # slack variable for determining correct cell
#param lambda; # heading change penalty factor
param d := 0.8*sqrt(dX^2+dY^2)+1e-5; #travel distance
param M := 10; # big-M for unity(-1-norm) velocity constraint >> 1 

set V := {1..Ny, 1..Nx}; # set of nodes
#set A{(iy,ix) in V} := {(vy,vx) in V: vy-1<=iy<=vy+1 && vx-1<=ix<=vx+1} diff {(ix,iy)}; # set of neighbors

var z{t in 0..Nh, i in 1..2} >= -1, <= 1;  
var yz{t in 0..Nh, i in 1..4} binary;
var x{t in 0..Nh} = if t = 0 then x0 else x[t-1] + d*z[t-1,1];
var y{t in 0..Nh} = if t = 0 then y0 else y[t-1] + d*z[t-1,2];
var cellX{t in 0..Nh} integer;
var cellY{t in 0..Nh} integer;
var yb{t in 0..Nh, (iy,ix) in V} = if iy=cellY[t] and ix=cellX[t] then 1 else 0; #binary
#var dx{t in 1..Nh} = if x[t]>=x[t-1] then x[t]-x[t-1] else x[t-1]-x[t]; # absolute change in x-position
#var dy{t in 1..Nh} = if y[t]>=y[t-1] then y[t]-y[t-1] else y[t-1]-y[t]; # absolute change in y-position
var dz{t in 1..Nh-1, i in 1..2} = if z[t,i]>=z[t-1,i] then z[t,i]-z[t-1,i] else z[t-1,i]-z[t,i]; # absolute change in unit velocity
var dxt = if x[Nh]>=xT then x[Nh]-xT else xT-x[Nh]; # absolute final distance to target in x-position
var dyt = if y[Nh]>=yT then y[Nh]-yT else yT-y[Nh]; # absolute final distance to target in y-position

# Objective Function
minimize cost: 1e-6*(dxt + dyt) + sum{t in 1..Nh} (sum{(iy,ix) in V} yb[t,iy,ix]*Z[iy,ix]); # (dx[t] + dy[t])

# Constraints
#subject to kinematics11{t in 1..Nh-1}: dz[t,1] <= dx[t]/U *rmax;
#subject to kinematics12{t in 1..Nh-1}: dz[t,2] <= dy[t]/U *rmax;
#subject to kinematics11{t in 1..Nh-1}: dz[t,2] <= dz[t,1]; #restrict heading change between -45 and 45deg

#subject to unitSquare1{t in 0..Nh}: - z[t,1] - z[t,2] <= -1 + M*yz[t,1];
#subject to unitSquare2{t in 0..Nh}: - z[t,1] + z[t,2] <= -1 + M*yz[t,2];
#subject to unitSquare3{t in 0..Nh}:   z[t,1] + z[t,2] <= -1 + M*yz[t,3];
#subject to unitSquare4{t in 0..Nh}:   z[t,1] - z[t,2] <= -1 + M*yz[t,4];
#subject to inQuadrant{t in 0..Nh}: sum{i in 1..4} yz[t,i] <= 3;

subject to inCellX1{t in 0..Nh}: cellX[t] >= x[t]/dX;
subject to inCellX2{t in 0..Nh}: cellX[t] <= x[t]/dX + s;
subject to inCellY1{t in 0..Nh}: cellY[t] >= y[t]/dY;
subject to inCellY2{t in 0..Nh}: cellY[t] <= y[t]/dY + s;
subject to workspaceX{t in 0..Nh}: 0 <= x[t] <= Lx;
subject to workspaceY{t in 0..Nh}: 0 <= y[t] <= Ly;