function [b] = blending_function(n,s)
        %[b, dot_b, ddot_b, dddot_b]
    % Calculates the blending functions up to (derivative) order 3 of a Bezier
    % curve curve with n control points 
    %
    % Input: n = Number of control points (4, 6, 8, 10). s = path parameter
    % Output: Blending functions up to (derivative) order 3.
    %
    % Magnus Knaedal 20.09.2019
    % Modified to be compatibile with Simulink: Caroline Fleischer, 18.02.2020


    % Bezier degree
    n_deg = n - 1;

    % Matrix containing elements.
    M = zeros(n); 
    sigma = zeros(1,n_deg+1);

    % Factorial function.
    for i = 0:1:n_deg
        sigma(i+1) = factorial(n_deg)/(factorial(i)*factorial(n_deg-i));  
    end

    for i = 0:1:n_deg
        for j = 0:1:n_deg
            if (i - j < 0)
                M(i+1, j+1) = 0;
            else
                M(i+1, j+1) = (-1)^(i-j) * sigma(i+1) * ...
                    (factorial(i)/(factorial(j)*factorial(i-j)));
            end

        end
    end

    t = s;
    if n == 4
        poly = [1 t t^2 t^3];    
        dot_poly = [0 1 2*t 3*t^2];
        ddot_poly = [0 0 2 6*t];
        dddot_poly = [0 0 0 6];
    elseif n == 6        
        poly = [1 t t^2 t^3 t^4 t^5];    
        dot_poly = [0 1 2*t 3*t^2 4*t^3 5*t^4];
        ddot_poly = [0 0 2 6*t 12*t^2 20*t^3];
        dddot_poly = [0 0 0 6 24*t 60*t^2];
    elseif n == 8
        poly =       [1 t t^2 t^3    t^4    t^5     t^6     t^7];    
        dot_poly =   [0 1 2*t 3*t^2  4*t^3  5*t^4   6*t^5   7*t^6];
        ddot_poly =  [0 0 2   6*t    12*t^2 20*t^3  30*t^4  42*t^5];
        dddot_poly = [0 0 0   6      24*t   60*t^2  120*t^3 210*t^4];
    else %n == 10
        poly = [1 t t^2 t^3 t^4 t^5 t^6 t^7 t^8 t^9];    
        dot_poly = [0 1 2*t 3*t^2 4*t^3 5*t^4 6*t^5 7*t^6 8*t^7 9*t^8];
        ddot_poly = [0 0 2 6*t 12*t^2 20*t^3 30*t^4 42*t^5 56*t^6 72*t^7];
        dddot_poly = [0 0 0 6 24*t 60*t^2 120*t^3 210*t^4 336*t^5 504*t^6];
    end
    
    b = [poly*M; dot_poly*M; ddot_poly*M; dddot_poly*M];
    
    %dot_b = dot_poly*M;
    %ddot_b = ddot_poly*M;
    %dddot_b = dddot_poly*M;
        
end