% Function evolving the neural network landscape based on the shunting 
% equation of Yang & Meng (2001)

%       N     = number of neurons
%       dT    = sample time
%       ups   = external input at each neuron position
%       A     = adjacency matrix
%       W     = connection weight matrix
%       z_old = previous neural activity
%       z_new = new neural activity
%       a     = passive decay rate (shunting eq.)
%       bu    = upper bound for neural activity (shunting eq.)            
%       bl    = lower bound for neural activity (shunting eq.)

function z_new = shuntingEq(N, dT, ups, A, W, z_old, a, bu, bl)
    
    z_new = z_old;
    for i = 1 : N
        ups_p   = max(0, ups(i));
        ups_m   = max(0,-ups(i));
        excite  = 0.0;
        % Indices of neighbouring neurons; length(col)=number of neighbors
        [~,col] = find(A(i,:));
        % Sum of exitatory input from lateral connections
        for jj = 1 : length(col)      
            z_m    = max(0,-z_old(col(jj)));
            excite = excite + W(i,col(jj))*z_m; 
        end
        % Shunting equation with Euler's method
        z_new(i) = z_old(i) + dT*(-a*z_old(i) - ...
            (bl+z_old(i))*(ups_m+excite) + (bu-z_old(i))*ups_p); 
    end
    
end