% Creates a video named 'sim_CSE1.avi'

v = VideoWriter('sim_CSE1.avi','Uncompressed AVI'); 
v.FrameRate = 2*200*h; open(v); % Create a movie file and open it
encounter = {'SO','HO','GW','OTp','OTs'};
for vf = 1:200:length(eta)
    figure(101); clf; hold on
    plot(path.signals.values(:,2,vf), path.signals.values(:,1,vf),'.',...
        'MarkerSize',14,'Color',gray,...
        'DisplayName','planned waypoints'); % Planned WPs
    plot([eta0(2);WP(1:vf,2)],[eta0(1);WP(1:vf,1)],'.','MarkerSize',14,...
        'Color',green,'DisplayName','waypoints') % Waypoints
    plot(y(idx_so),x(idx_so),'ks','MarkerFaceColor','k',...
        'DisplayName','static obstacles'); % Static obstacles
    plot(y(idx_t),x(idx_t),'p','Color',red,'MarkerFaceColor',red,...
        'MarkerSize',8,'DisplayName','target'); % Target
    plot(etad(1:vf,2), etad(1:vf,1),'-','Color',green,...
        'DisplayName','B\''ezier path') % Desired Bezir path
    plot(eta(1:vf,2), eta(1:vf,1),'-k',...
        'DisplayName','vessel tracks') % Vessel tracks
    plot(eta_obj(1:vf,2), eta_obj(1:vf,1),'-.','Color',orange,...
        'DisplayName','object tracks') % Moving object
    [yGrid,xGrid] = meshgrid(0:dY:Ly,0:dX:Lx); % Grid
    plot(yGrid,xGrid,':','Color',gray,'HandleVisibility','off');
    plot(yGrid',xGrid',':','Color',gray,'HandleVisibility','off');
    axis([0 Ly 0 Lx]), axis equal tight manual, box on
    % Plot vessel
    R0 = [cos(eta(vf,3)) -sin(eta(vf,3)); 
        sin(eta(vf,3))  cos(eta(vf,3))];
    ship = R0*[6*L/9 5*L/9 3*L/9 -5*L/9 -5*L/9 3*L/9 5*L/9 6*L/9;
        0 1*L/9 2*L/9 2*L/9 -2*L/9 -2*L/9 -1*L/9 0];
    plot(eta(vf,2)+ship(2,:), eta(vf,1)+ship(1,:),'k',...
        'HandleVisibility','off');
    Rd = rd * [cos(theta); sin(theta)] + eta(vf,1:2)'; % Detection region
    fill(Rd(2,:),Rd(1,:),blue,'LineStyle','none','FaceAlpha',.05,...
        'DisplayName','detection region');
    fill(obj_def(2,:)+eta_obj(vf,2),obj_def(1,:)+eta_obj(vf,1),orange,...
        'FaceAlpha',.2,'EdgeColor',orange,...
        'HandleVisibility','off'); % Object position
    fill(avoidA(2,:,vf),avoidA(1,:,vf),orange,'FaceAlpha',.1,'Linestyle',...
        'none','DisplayName','avoidance region'); % Avoidance region 
    annotation(figure(101),'textbox',[.4 .775 .1 .1],'String',... 
        'Encounter status:'+" "+encounter(status(vf)),... 
        'LineStyle','none','FontSize',9); % Type of encounter
    legend('show','Location','southeastoutside')
    xlabel('y [m]'); xticks(dY:2*dY:Ly)
    ylabel('x [m]'); yticks(dX:2*dX:Lx)
    
    % Zoomed subplot: Create a new pair of axes inside current figure
    axes('position',[.70 .58 .2 .2]) %[left bottom width height]
    box on % put box around new pair of axes
    if vf == 1
        idxOfInterest = vf;
        prev_vf = vf;
        nextWP = WP(vf,:);
    elseif sum(WP(vf,:)==nextWP) ~= 2
        prev_vf = vf-200;
        nextWP = WP(vf,:);
        idxOfInterest = prev_vf:vf;
    else
        idxOfInterest = prev_vf:vf; 
    end
    plot(eta(idxOfInterest,2),eta(idxOfInterest,1),'-k'); hold on 
    plot(etad(idxOfInterest,2),etad(idxOfInterest,1),'-','Color',green)
    plot(WP(idxOfInterest,2),WP(idxOfInterest,1),'.','MarkerSize',14,...
        'Color',green)
    % Plot vessel
    R0 = [cos(eta(vf,3)) -sin(eta(vf,3)); 
        sin(eta(vf,3))  cos(eta(vf,3))];
    ship = R0*[6*L/9 5*L/9 3*L/9 -5*L/9 -5*L/9 3*L/9 5*L/9 6*L/9;
        0 1*L/9 2*L/9 2*L/9 -2*L/9 -2*L/9 -1*L/9 0];
    plot(eta(vf,2)+ship(2,:), eta(vf,1)+ship(1,:),'k','Linewidth',1.5)
    yAx = get(gca,'XLim'); xAx = get(gca,'YLim'); 
    yAxTick = round(linspace(yAx(1),yAx(2),9)); set(gca,'XTick',yAxTick(5));
    xAxTick = round(linspace(xAx(1),xAx(2),9)); set(gca,'YTick',xAxTick(5));
    axis equal %manual, xlim([eta(vf,2)-dY/2 eta(vf,2)]) 
        
    hold off;
    
    % Capture frames for a movie of the vehicle tracks
    writeVideo(v,getframe(figure(101)));
end

close(v); % Close the movie file